<?php

/*
 * This script scans php scripts in a directory and shows php files with
 * warnings. In the source view the deprecated functions will be
 * highlighted.
 * http://php.net/manual/en/migration53.deprecated.php
 * v1.7
 * 
 * License: GPL 3.0 - http://www.gnu.org/licenses/gpl-3.0.html
 * THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE 
 * LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR 
 * OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, 
 * EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE 
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. 
 * SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY 
 * SERVICING, REPAIR OR CORRECTION.
 * 
 * --------------------------------------------------------------------------------
 * project home:
 * http://www.axel-hahn.de/php_deprecated.php
 * 
 * Download newest version:
 * http://www.axel-hahn.de/axel/download/find_php53_deprecated.zip
 * --------------------------------------------------------------------------------
 * 
 * --- history:
 * 2014-11-30  1.7  
 * 2014-11-30  1.6  use json file for config of deprecated items
 * 2012-05-03  1.5  show dirs and files in sorted order
 * 2012-04-24  1.4  support of deprecated ini values; set php version 5.3 | 5.4
 * 2012-04-24  1.3  execute search only if the script was opened directly
 * 2012-04-24  1.2  written as php class
 * 2012-04-23  1.1  fixes: search for magic_quotes_runtime; 
 *                  improvements: - conclusion for found errors
 *                                - in some functions: how to replace the code
 * 2012-04-22  1.0  first version
 */

class ScanDeprecated {

    private $sPrjHome = 'http://www.axel-hahn.de/php_deprecated.php';
    private $sWebsite = 'www.axel-hahn.de';
    private $sAbout = 'PHP find deprecated code :: v1.7 :: (c) 2012-2014 Axel Hahn';
    private $sHtmlTitle = 'PHP :: find deprecated code';

    /**
     * compatibility level: php version
     * @var string
     */
    var $sPHPVersion = '5.3';

    /**
     * working directory where the scan starts
     * @var type 
     */
    var $sWorkingDir = false;

    /**
     * flag: enable recursive scan
     * @var bool
     */
    var $bRecursive = false;

    /**
     * filename to view its source
     * @var string 
     */
    var $sSourcefile = false;

    /**
     * array of depricated elements
     * @var array
     */
    private $aOldFunctions = array();

    /**
     * array of REGEX to search deprecated elements (will be filled in parseConfigArray)
     * @var array
     */
    private $aSearch = array();

    /**
     * array of hints for each deprecated element (will be filled in parseConfigArray)
     * @var array
     */
    private $aHints = array();
    
    /**
     * array of extensions to scan
     * @var type 
     */
    private $_aExtensionsToScan=array("php","php3","php4","php5","phar");

    /*
     * ======================================================================
     * 
     * INIT
     * 
     * ======================================================================
     */

    /**
     * constructor 
     */
    public function __construct() {

        $this->parseConfigArray();
        $this->parseURLParams();
    }

    /**
     * parse GET params and set object variables
     * @return boolean 
     */
    private function parseURLParams() {
        if ($_GET) {

            if (array_key_exists("s", $_GET))
                $this->setWorkingDir($_GET["s"]);
            if (array_key_exists("r", $_GET))
                $this->setFlagRecursive($_GET["r"]);
            if (array_key_exists("f", $_GET))
                $this->setSourcefile($_GET["f"]);
            if (array_key_exists("v", $_GET))
                $this->setPhpVersion($_GET["v"]);
        }
        return true;
    }

    /**
     * helper function: rewrite config array with deprecated functions
     */
    private function parseConfigArray($sCfgfile = "deprecated.json") {
        $this->aSearch = array();
        $this->aHints = array();

        if (!file_exists($sCfgfile)) {
            die("ERROR: $sCfgfile does not exist.");
        }
        $this->aOldFunctions = json_decode(file_get_contents($sCfgfile), true);
        if (!is_array($this->aOldFunctions)) {
            print_r($this->aOldFunctions);
            die("ERROR: $sCfgfile is not a valid JSON :-/");
        }
        // $sData=json_encode($this->aOldFunctions); file_put_contents($sCfgfile, $sData);
        
        // loop over versions
        foreach ($this->aOldFunctions as $version => $aData) {
            if ($version <= $this->sPHPVersion) {

                // loop over ini settings
                if (array_key_exists("ini", $aData)) {
                    foreach ($aData["ini"] as $key => $a) {
                        $this->aSearch[] = '/(ini\_.*\()(.*' . $a[0] . '.)(.*[\,\ \)])/U';
                        $sHint = '$1<span class="spanerror">';
                        if (array_key_exists(2, $a)) {
                            $sTo = strip_tags(preg_replace('/\(.*$/', '', $a[2])); // cut "(" and behind
                            $sHint.=$this->getHelpLink($sTo, $s . '(<span>example how to modify the source:<br><br>    FROM: ' . $a[0] . '<br>      TO: ' . $a[2] . '<br><br>Click to show help for function <em>' . $sTo . '.</em></span>');
                        } else {
                            $sHint.='$2';
                        }
                        $sHint.='</span><span class="spanhint"> &laquo INI value <strong>&quot;' . $a[0] . '&quot;</strong> is deprecated in php ' . $version . '</span>$3';
                        $this->aHints[] = $sHint;
                    }
                }

                // loop over functions
                if (array_key_exists("functions", $aData)) {
                    foreach ($aData["functions"] as $key => $a) {
                        $s = preg_replace('/\(.*$/', '', $a[0]); // cut "(" and behind
                        $this->aSearch[] = '/([!\W])(' . $s . '\()/';
                        $sHint = '$1<span class="spanerror">';
                        if (array_key_exists(2, $a)) {
                            $sTo = strip_tags(preg_replace('/\(.*$/', '', $a[2])); // cut "(" and behind
                            $sHint.=$this->getHelpLink($sTo, $s . '(<span>example how to modify the source:<br><br>    FROM: ' . $a[0] . '<br>      TO: ' . $a[2] . '<br><br>Click to show help for new function <em>' . $sTo . '.</em></span>');
                        } else {
                            $sHint.=$this->getHelpLink($s, $s . '(<span>Sorry I have no hint how to modify <em>'.$s.'</em>.<br>Click to show help for depricated function <em>' . $s . '.</em></span>');
                        }
                        $sHint.='</span><span class="spanhint"> &laquo function is deprecated in php ' . $version . ' - ' . $a[1] . '</span>';
                        $this->aHints[] = $sHint;
                    }
                }
            }// if 
        } // foreach
        // echo "<pre>" . print_r($this->aSearch, true) . "</pre>";
    }

    /**
     * run actions (scan direoctory or scan a file) and show output
     * as an html document.
     */
    public function renderHtml() {

        $this->showHtmlHead();

        if ($this->sSourcefile) {
            $this->showSourcefile();
        } else {
            if (!$this->sWorkingDir) {
                $this->setWorkingDir();
            }
            if ($this->sWorkingDir) {
                $this->showHtmlForm();
                echo '<div id="dirs">';
                $sUpdir = dirname($this->sWorkingDir);
                $sUpdir = str_replace('\\', '/', $sUpdir);
                $sUpdir = str_replace('//', '/', $sUpdir);
                if ($sUpdir == $this->sWorkingDir){
                    $sUpdir = false;
                }
                if ($sUpdir) {
                    $sDir = '';
                    echo '<div id="breadcrumb">';
                    foreach (preg_split('#/#', dirname($this->sWorkingDir)) as $sDirname) {
                        $sDir.=$sDirname . "/";
                        echo "<a href='?s=$sDir&amp;v=$this->sPHPVersion' "
                            . "title ='up to $sDir'"
                                . ">$sDirname</a> / ";
                    }
                    echo basename($this->sWorkingDir);
                    echo "</div>";
                } else {
                    echo "<br>";
                }

                $iCount=$this->scanMyDir();
                if (!$iCount){
                    echo "WARNING: No subdir or file was found.</li>";
                }
                echo '</div>';
            } else {
                $this->setWorkingDir();
                $this->showHtmlForm();
            }
        }
        $this->showHtmlFooter();
    }

    /*
     * ======================================================================
     * 
     * SEVERAL ACTIONS
     * 
     * ======================================================================
     */

    /**
     * Check: Does a given source string have deprecated elements?
     * @param string $sSource
     * @return string  html code; false if no deprecated function was found
     */
    function hasDeprecated($sSource) {
        $sSource = htmlentities($sSource);
        $sReplaced = preg_replace($this->aSearch, $this->aHints, $sSource);
        if ($sSource == $sReplaced) {
            return false;
        }
        return $sReplaced;
    }

    /**
     * get url for php function help page
     * @param string $sFunction function name
     * @return string url
     */
    function getHelpURL($sFunction) {
        return 'http://php.net/manual/en/function.' . str_replace('_', '-', $sFunction) . '.php';
    }

    /**
     * generate a link to the php.net help page
     * @param string $sFunction  function name
     * @param string $LinkText   linktext(optional; default is function name)
     * @return string html code
     */
    function getHelpLink($sFunction, $LinkText = '') {
        if (!$LinkText)
            $LinkText = $sFunction;
        return '<a href="' . $this->getHelpURL($sFunction) . '" target="php">' . $LinkText . '</a>';
    }

    /**
     * directory scan 
     * @param string $sDir  directory to scan. if empty it starts with $this->sWorkingDir
     * @return boolean (dummy)
     */
    function scanMyDir($sDir = false) {
        if (!$sDir) {
            $sDir = $this->sWorkingDir;
        }

        $sDir = str_replace('//', '/', $sDir);
        if (!(file_exists($sDir))) {
            echo "<span class='error'>ERROR: Cannot open directory - [$sDir]</span>";
            return;
        }
        $aDirs = array();
        $aFiles = array();
        foreach (scandir($sDir) as $entry) {
            $sEntry = $sDir . "/" . $entry;
            $sEntry = str_replace('//', '/', $sEntry);
            if (is_dir($sEntry) && $entry != '.' && $entry != '..') {
                $aDirs[] = $sEntry;
            }
            if (file_exists($sEntry) && !is_dir($sEntry)) {
                $aFiles[] = $sEntry;
            }
        }
        // echo "<pre>" . print_r($aDirs, true).print_r($aFiles, true)."</pre>";

        echo ($this->bRecursive)?'<ul>':'<ul style="float: left; width: 20em;">';
        foreach ($aDirs as $entry) {
            echo "<li class=\"dir\">► DIR: <strong><a href='?s=$entry&amp;r=$this->bRecursive&amp;v=$this->sPHPVersion' title='scan $sEntry'>" . basename($entry) . "</a></strong>\n";
            if ($this->bRecursive) {
                echo "<ul>\n";
                $this->scanMyDir($entry);
                echo "</ul>\n";
            }
            echo "</li>\n";
        }
        echo ($this->bRecursive)?'':'</ul><ul style="margin-left: 22em;">';
        
        foreach ($aFiles as $entry) {
            
            $ext = pathinfo($entry, PATHINFO_EXTENSION);
            if (array_search($ext, $this->_aExtensionsToScan)!==false){
                $sContent = @file_get_contents($entry);
                if ($this->hasDeprecated($sContent)) {
                    // found something....
                    echo "<li class='error'>‼ <a href='?f=$entry&amp;v=$this->sPHPVersion' title='show source with marked errors'>" . basename($entry) . "</a> (ERROR)</li>\n";
                } else {
                    echo "<li class='ok'>☺ " . basename($entry) . " (OK)</li>\n";
                }
            } else {
                echo "<li class='skip'>░ " . basename($entry) . " (SKIP)</li>\n";
            }
        }
        // echo "</ul></li>";
        echo "</ul>";

        return (count($aDirs) + count($aFiles));
    }

    /**
     * show source of a file with list of found errors as html
     * @return none
     */
    public function showSourcefile() {
        echo '<div id="divform">';
        echo "<br> <a href='#' onclick='javascript: history.back(); return false;' class='btn'>&laquo; back</a><br><br>";
        echo '</div>';
        echo '<div id="filename">';
        echo "<strong>Filename</strong>: $this->sSourcefile<br>";
        echo '</div>';
        if (!file_exists($this->sSourcefile)) {
            echo "<div class='error'>ERROR: file was not found.</div><br>";
        } else {
            $sContent = file_get_contents($this->sSourcefile);
            // $sContent = highlight_file($this->sSourcefile);
            
            $sReplaced = $this->hasDeprecated($sContent);
            $sErrors = '';

            if ($sReplaced) {
                echo "<div class='error'>WARNING: depricated elements (up to PHP version $this->sPHPVersion) have been found. They are highlighted in the source below.";
                // $sContent = $sReplaced;
                $sContent = '';
                // $aTmp = preg_split("/\n/", $sContent);
                $aTmp = file($this->sSourcefile);
                $i = 0;
                foreach ($aTmp as $sLine) {
                    $i++;
                    $sLine2 = $this->hasDeprecated($sLine);
                    if (!$sLine2) {
                        $sLine2 = htmlentities($sLine);
                    } else {
                        $sErrors.='<li><a href="#line' . $i . '" title="jump to line ' . $i . '">line ' . $i . '</a>: ' . trim($sLine2) . '</li>';
                    }
                    $sContent.="<li id='line$i'>" . $sLine2 . "</li>";
                }
            } else {
                echo "<div class='ok'>OK: no depricated elements have been found.";
                $sContent = htmlentities($sContent);
            }
            if ($sErrors)
                echo "<br><br>Found errors:<pre><ol>$sErrors</ol></pre>Complete source:";
            echo "<pre><ol>$sContent</ol></pre></div><br>";
            echo "&nbsp;<a href='#' onclick='javascript: history.back(); return false;' class='btn'>&laquo; back</a><br><br>";
            echo "<br><br><br><br><br><br><br>";
        }
    }

    /**
     * write beginning of html document
     * @return none
     */
    public function showHtmlHead() {
        echo "<!doctype html>
        <html><!--
        **********************************************************************
      
        $this->sAbout
                
        $this->sPrjHome
      
        **********************************************************************
    --><head><title>$this->sHtmlTitle</title>
        <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />
        <meta name='author' content='Axel Hahn'>
        <meta name='Robots' content='noindex, nofollow'>

        <style>
            a{color:#33c; text-decoration: none; }
            a:hover{text-decoration: underline; }
            pre a{color:#33c; border-bottom: 1px dotted;text-decoration: none;}
            a span{display: none; text-decoration: none; position: absolute; background: #fff; padding: 5px; box-shadow: 3px 5px 7px rgba(0,0,0,0.4); width: 90%;}
            a:hover span{display: block; }
            a:hover span .r{color:#f0c; display: inline; width: auto; position: relative;background: none; padding: 0;box-shadow: none;}
            body{background:#fff; color: #333; font-family: arial; margin: 0; padding: 0;}
            h1{color:#669; background:#333; background: linear-gradient(#222,#334); margin: 0; }
            ul{margin-top: 0em; width: 95%;}
            
            #divhelper{z-index: 1000; position: fixed; top: 1em; right: 1em; background:#eee; padding: 1em; min-width: 10em; display: none; border: 2px solid #ccc;}
            #divhelper li{list-style: none;}
            #divform{background:#666; background: linear-gradient(#444,#666); padding: 0.3em; color:#ccc;}
            #divform .options{background: none; float: left; padding: 0.3em;color:#ddd; }
            
            
            input{padding: 7px; border-radius: 3px; border: 2px solid #ddd;}
            pre{background:#eee; opacity: 0.8; color:#666; padding: 10px;}
            
            .btn{border: 2px solid #888; background:#889; padding: 0.2em; border-radius: 0.4em;
                text-decoration: none;
                color:#eee;
                box-shadow: 0 0 1em #222;
            }
            .btn:hover{border: 2px solid #aac;}
            #breadcrumb, #filename{background: #cce;padding: 0 0.5em;
                border-top: 1px solid #668;
                border-bottom: 1px solid #eee;
                box-shadow: 0 0 1em #888;
                margin-bottom: 1em;
            }
            #breadcrumb a{}
            #dirs{margin-bottom: 8em;}
            #dirs li{list-style: none;}
            #footer{
                color:#ccc; text-align: right; 
                font-size: 80%; 
                position:fixed; bottom: 0;
                background: #666; background: linear-gradient(#666,#444); 
                border-top: 1px solid #aaa;
                box-shadow: 0 -0.3em 1em #aaa;
                width: 100%;
            }
            #footer a{color:#aae;}

            pre{
                border-radius: 10px;
            }
            .dir, .ok, .error, .skip{padding: 3px;border-left: 1px dotted #ccc; margin-bottom: 2px;}
            .dir{background: #f0f0d0;}
            .submit{background: #aae; border: 1px solid #555;}
            .submit:hover{background: #99d;}
            .ok{color:#383; background:#ded;}
            .error{color:#833; background:#edd;}
            .skip{color:#999; background:#f8f8f8; }
            .spanerror{color:#800; background: #faa}
            .spanhint{color:#333; background: #fea}
        </style>
        <script src=\"jquery.js\" type=\"text/javascript\" ></script>
    </head><body>
    <div id=\"divhelper\"></div>
    <h1>$this->sHtmlTitle</h1>";
    }

    /**
     * write searchform as html
     * @return none
     */
    public function showHtmlForm() {
        echo '
            <div id="divform">
                <form action="?">
                                    
                    <div class="options">
                        <input type="checkbox" ' . $this->bRecursive . ' name="r" value="1" onchange="this.form.submit()"/> recursive scan
                        | 
                        PHP version: 
                        <select name="v" onchange="this.form.submit()">';
                        foreach ($this->aOldFunctions as $version => $aDummy) {
                            /*
                            echo '<input type="radio" name="v" value="' . $version . '"';
                            if ($version == $this->sPHPVersion)
                                echo ' checked ';
                            echo '/> ' . $version . '  ';
                             */
                            echo '<option value="' . $version . '"';
                            if ($version == $this->sPHPVersion)
                                echo ' selected="selected" ';
                            echo '> ' . $version . '</option>';
                        }
                        echo '
                        </select>
                    </div>
                    
                    <div style="float: left;">
                        <!-- Directory to scan:<br> -->
                        <input type="hidden" name="s" value="' . $this->sWorkingDir . '" size="75" />
                        <input type="submit" class="submit" value="go" />
                    </div>
                    <div style="clear: both;"></div>
                </form>
            </div>
            ';
    }

    /**
     * write html footer and close html document 
     * @return none
     */
    public function showHtmlFooter() {
        echo "<div id='footer'>
                <a href='#' style='left: 20px; float: left;'>jump to top</a>
                generated by " . __FILE__ . "<br>" . date("d.m.Y H:i:s") . "<br>
                <em>$this->sAbout</em> - <a href='$this->sPrjHome' title='visit project home' target='_blank'>$this->sWebsite</a>
              </div>
              <script>
                $( document ).ready(function() {
                    initPage();
                });
                
                function mkStatuslink(sClass,sLabel){
                    var s='';
                    if ($('li.'+sClass).length){
                        if(sClass!='dir'){
                            s+='<li class=\"'+sClass+'\"><a href=\"#\" title=\"toggle '+sLabel+'\" onclick=\"$(\'#dirs li.'+sClass+'\').toggle(); return false\">';
                            s+=$('li.'+sClass).length+' '+sLabel;
                            s+='</a></li>';
                        } else {
                            s+=$('li.'+sClass).length+' '+sLabel+'<br>';
                        }
                    }
                    return s;
                }
                function initPage(){
                    var s='';
                    
                    if ($('#dirs').length){
                        var sErr=mkStatuslink('error', '‼ errors');

                        s+=mkStatuslink('dir', '► dirs');
                        s+=mkStatuslink('ok', '☺ ok');
                        s+=mkStatuslink('skip', '░ skip');
                        s+=sErr;
                        if (sErr){
                            s+='<br><a href=\"#\" onclick=\"$(\'#dirs li.ok\').hide();$(\'#dirs li.skip\').hide();$(\'#dirs li.error\').show();return false;\">show errors only</a>'
                        } else {
                            s+='... but no error :-)</a><br>';
                        }
                        if(s){
                            s='<strong>I found ...</strong><br><br>'+s;
                            s+='<br><a href=\"#\" onclick=\"$(\'#dirs li.ok\').show();$(\'#dirs li.skip\').show();$(\'#dirs li.error\').show(); return false;\">show all</a>'
                            $('#divhelper').html(s);
                            $('#divhelper').fadeIn();
                        } else {
                            $('#divhelper').hide();
                        }
                    }
                }
            </script>
        </body></html>";
    }

    /*
     * ======================================================================
     * 
     * SETTER
     * 
     * ======================================================================
     */

    /**
     * set flag for recursive scan
     * @param bool $bFlag
     * @return bool
     */
    public function setFlagRecursive($bFlag = false) {
        if ($bFlag == 1 || $bFlag == 'checked') {
            $this->bRecursive = 'checked';
        } else {
            $this->bRecursive = false;
        }
        return $this->bRecursive;
    }

    /**
     * set a php version (5.3 or 5.4)
     * @param string  $s  version
     * @return string current version
     */
    public function setPhpVersion($s = '') {
        if ($s && array_key_exists($s, $this->aOldFunctions)) {
            $this->sPHPVersion = $s;
            $this->parseConfigArray(); // reinit what stuff to parse
        }
        return $this->sPHPVersion;
    }

    /**
     * set a sourcefile to view
     * @param string  $s  path
     * @return string current filename with full path
     */
    public function setSourcefile($s = '') {
        if ($s) {
            $this->sSourcefile = $s;
        } else {
            $this->sSourcefile = false;
        }
        $this->sSourcefile = str_replace('\\', '/', $this->sSourcefile);
        $this->sSourcefile = str_replace('//', '/', $this->sSourcefile);
        return $this->sSourcefile;
    }

    /**
     * set a working directory (starting point of a scan)
     * @param string  $s  path
     * @return string current working directory
     */
    public function setWorkingDir($s = '') {
        if ($s) {
            $this->sWorkingDir = $s;
        } else {
            if ($_SERVER && $_SERVER["SCRIPT_FILENAME"]) {
                $this->sWorkingDir = dirname($_SERVER["SCRIPT_FILENAME"]);
            }
        }
        $this->sWorkingDir = str_replace('\\', '/', $this->sWorkingDir);
        $this->sWorkingDir = str_replace('//', '/', $this->sWorkingDir);
        return $this->sWorkingDir;
    }

}

/*
 * ======================================================================
 * 
 * MAIN
 * 
 * ======================================================================
 */

// if this file was opened directly then execute renderHtml()
if ($_SERVER && basename($_SERVER["SCRIPT_FILENAME"]) == basename(__FILE__)) {
    $oScanner = new ScanDeprecated();
    // $oScanner->setPhpVersion('5.4');
    $oScanner->renderHtml();
    unset($oScanner);
}

// ----------------------------------------------------------------------